/*
 * The Clear BSD License
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 * that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef _PIN_MUX_H_
#define _PIN_MUX_H_

/***********************************************************************************************************************
 * Definitions
 **********************************************************************************************************************/

/*! @brief Direction type  */
typedef enum _pin_mux_direction
{
    kPIN_MUX_DirectionInput = 0U,        /* Input direction */
    kPIN_MUX_DirectionOutput = 1U,       /* Output direction */
    kPIN_MUX_DirectionInputOrOutput = 2U /* Input or output direction */
} pin_mux_direction_t;

/*!
 * @addtogroup pin_mux
 * @{
 */

/***********************************************************************************************************************
 * API
 **********************************************************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif


/*! @name PORTE22 (number 11), J3[2]/J20A[B52]/J20A[B29]/UART2_TX
  @{ */
#define BOARD_DEBUG_UART_TX_PERIPHERAL UART2                           /*!<@brief Device name: UART2 */
#define BOARD_DEBUG_UART_TX_SIGNAL TX                                  /*!<@brief UART2 signal: TX */
#define BOARD_DEBUG_UART_TX_PORT PORTE                                 /*!<@brief PORT device name: PORTE */
#define BOARD_DEBUG_UART_TX_PIN 22U                                    /*!<@brief PORTE pin index: 22 */
#define BOARD_DEBUG_UART_TX_PIN_NAME UART2_TX                          /*!<@brief Pin name */
#define BOARD_DEBUG_UART_TX_LABEL "J3[2]/J20A[B52]/J20A[B29]/UART2_TX" /*!<@brief Label */
#define BOARD_DEBUG_UART_TX_NAME "DEBUG_UART_TX"                       /*!<@brief Identifier name */
                                                                       /* @} */

/*! @name PORTE23 (number 12), J5[2]/J20A[B30]/J20A[A35]/UART2_RX
  @{ */
#define BOARD_DEBUG_UART_RX_PERIPHERAL UART2                           /*!<@brief Device name: UART2 */
#define BOARD_DEBUG_UART_RX_SIGNAL RX                                  /*!<@brief UART2 signal: RX */
#define BOARD_DEBUG_UART_RX_PORT PORTE                                 /*!<@brief PORT device name: PORTE */
#define BOARD_DEBUG_UART_RX_PIN 23U                                    /*!<@brief PORTE pin index: 23 */
#define BOARD_DEBUG_UART_RX_PIN_NAME UART2_RX                          /*!<@brief Pin name */
#define BOARD_DEBUG_UART_RX_LABEL "J5[2]/J20A[B30]/J20A[A35]/UART2_RX" /*!<@brief Label */
#define BOARD_DEBUG_UART_RX_NAME "DEBUG_UART_RX"                       /*!<@brief Identifier name */
                                                                       /* @} */

/*! @name PORTA12 (number 28), J22[17]/J20A[B38]/J20A[A53]/J23[2]/LED1
  @{ */
#define BOARD_LED1_PERIPHERAL GPIOA                                /*!<@brief Device name: GPIOA */
#define BOARD_LED1_SIGNAL GPIO                                     /*!<@brief GPIOA signal: GPIO */
#define BOARD_LED1_GPIO GPIOA                                      /*!<@brief GPIO device name: GPIOA */
#define BOARD_LED1_GPIO_PIN 12U                                    /*!<@brief PORTA pin index: 12 */
#define BOARD_LED1_PORT PORTA                                      /*!<@brief PORT device name: PORTA */
#define BOARD_LED1_PIN 12U                                         /*!<@brief PORTA pin index: 12 */
#define BOARD_LED1_CHANNEL 12                                      /*!<@brief GPIOA GPIO channel: 12 */
#define BOARD_LED1_PIN_NAME PTA12                                  /*!<@brief Pin name */
#define BOARD_LED1_LABEL "J22[17]/J20A[B38]/J20A[A53]/J23[2]/LED1" /*!<@brief Label */
#define BOARD_LED1_NAME "LED1"                                     /*!<@brief Identifier name */
                                                                   /* @} */

/*! @name PORTA13 (number 29), J22[18]/J20A[B33]/J20A[B37]/J18[2]/LED2
  @{ */
#define BOARD_LED2_PERIPHERAL GPIOA                                /*!<@brief Device name: GPIOA */
#define BOARD_LED2_SIGNAL GPIO                                     /*!<@brief GPIOA signal: GPIO */
#define BOARD_LED2_GPIO GPIOA                                      /*!<@brief GPIO device name: GPIOA */
#define BOARD_LED2_GPIO_PIN 13U                                    /*!<@brief PORTA pin index: 13 */
#define BOARD_LED2_PORT PORTA                                      /*!<@brief PORT device name: PORTA */
#define BOARD_LED2_PIN 13U                                         /*!<@brief PORTA pin index: 13 */
#define BOARD_LED2_CHANNEL 13                                      /*!<@brief GPIOA GPIO channel: 13 */
#define BOARD_LED2_PIN_NAME PTA13                                  /*!<@brief Pin name */
#define BOARD_LED2_LABEL "J22[18]/J20A[B33]/J20A[B37]/J18[2]/LED2" /*!<@brief Label */
#define BOARD_LED2_NAME "LED2"                                     /*!<@brief Identifier name */
                                                                   /* @} */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void BOARD_InitPins(void);

/*! @name PORTE0 (number 1), J29[2]/J20A[A8]//RTC_CLKOUT/ACC_I2C1_SDA/LCD_P48
  @{ */
/*!
 * @brief Device name: I2C1 */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_PERIPHERAL I2C1
/*!
 * @brief I2C1 signal: SDA */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_SIGNAL SDA
/*!
 * @brief PORT device name: PORTE */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_PORT PORTE
/*!
 * @brief PORTE pin index: 0 */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_PIN 0U
/*!
 * @brief Pin name */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_PIN_NAME I2C1_SDA
/*!
 * @brief Label */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_LABEL "J29[2]/J20A[A8]//RTC_CLKOUT/ACC_I2C1_SDA/LCD_P48"
/*!
 * @brief Identifier name */
#define BOARD_I2C_CONFIGUREPINS_ACC_SDA_NAME "ACC_SDA"
/* @} */

/*! @name PORTE1 (number 2), J26[2]/J20A[A7]/ACC_I2C1_SCL/LCD_P49
  @{ */
/*!
 * @brief Device name: I2C1 */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_PERIPHERAL I2C1
/*!
 * @brief I2C1 signal: SCL */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_SIGNAL SCL
/*!
 * @brief PORT device name: PORTE */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_PORT PORTE
/*!
 * @brief PORTE pin index: 1 */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_PIN 1U
/*!
 * @brief Pin name */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_PIN_NAME I2C1_SCL
/*!
 * @brief Label */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_LABEL "J26[2]/J20A[A7]/ACC_I2C1_SCL/LCD_P49"
/*!
 * @brief Identifier name */
#define BOARD_I2C_CONFIGUREPINS_ACC_SCL_NAME "ACC_SCL"
/* @} */

/*!
 *
 */
void BOARD_I2C_ConfigurePins(void);

#if defined(__cplusplus)
}
#endif

/*!
 * @}
 */
#endif /* _PIN_MUX_H_ */

/***********************************************************************************************************************
 * EOF
 **********************************************************************************************************************/
